/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSTTY_INTERFACE_H
#define CONEXUSTTY_INTERFACE_H

#include <conexus/pointer.h>
#include <conexus/enums.h>

#include <conexus/serial_interface.h>

#include <termios.h>
#include <unistd.h>

namespace Conexus
{

  /**
   * @author Rick L Vinyard Jr
   */
  class TTYInterface: public virtual SerialInterface
  {
    public:

      typedef ConexusPointer<TTYInterface> pointer;

      TTYInterface() { }

      virtual ~TTYInterface() { }

      /**
       * waits until all output written to the object has been transmitted.
       */
      virtual void drain() = 0;

      /**
       * discards data received but not read
       */
      virtual void flush_input() = 0;

      /**
       * discards data written to the object but not transmitted
       */
      virtual void flush_output() = 0;

      /**
       * discards both data written to the object but not transmitted and data
       * received but not read
       */
      virtual void flush() = 0;

      /**
       * Suspends transmission of data by the object
       */
      virtual void suspend_output() = 0;

      /**
       * Resumes transmission of data by the object
       */
      virtual void restart_output() = 0;

      /**
       * Suspends reception of data by the object
       */
      virtual void suspend_input() = 0;

      /**
       * Resumes reception of data by the object
       */
      virtual void restart_input() = 0;

      /**
       * Sets the input line speed
       *
       * These are generally defined in /usr/include/bits/termios.h
       * The following is the recognized list:
       * - 0 (hang up)
       * - 50
       * - 75
       * - 110
       * - 134
       * - 150
       * - 200
       * - 300
       * - 600
       * - 1200
       * - 1800
       * - 2400
       * - 4800
       * - 9600
       * - 19200
       * - 38400
       * - 57600
       * - 115200
       * - 230400
       * - 460800
       * - 500000
       * - 576000
       * - 921600
       * - 1000000
       * - 1152000
       * - 1500000
       * - 2000000
       * - 2500000
       * - 3000000
       * - 3500000
       * - 4000000
       */
      virtual void set_input_speed( unsigned speed, SetOption option = SET_NOW ) = 0;

      /** Sets the output line speed */
      virtual void set_output_speed( unsigned speed, SetOption option = SET_NOW ) = 0;

      /** Sets both the input and output line speeds */
      virtual void set_speed( unsigned speed, SetOption option = SET_NOW ) = 0;

      /** Returns the current input line speed */
      virtual unsigned input_speed() = 0;

      /** Returns the current output line speed */
      virtual unsigned output_speed() = 0;

      /** Sets parity mode */
      virtual void set_parity( Parity parity, ParityError error = PARITY_ERROR_IGNORE, SetOption option = SET_NOW ) = 0;

      /** Returns the current parity mode */
      virtual Parity parity() = 0;

      /** Returns the current parity error mode */
      virtual ParityError parity_error() = 0;

      /** Get the current byte size in bits */
      virtual unsigned byte_size() = 0;

      /**
       * Sets the communication byte size
       * Valid values are:
       *  - 5 bits-per-byte
       *  - 6 bits-per-byte
       *  - 7 bits-per-byte
       *  - 8 bits-per-byte (default)
       * All other values are ignored.
       */
      virtual void set_byte_size( unsigned size, SetOption option = SET_NOW ) = 0;

      /** Get the number of stop bits (1 or 2) */
      virtual unsigned stop_bits() = 0;

      /** Sets the stop bits; only values of 1 or 2 will be processed, all others will be ignored. */
      virtual void set_stop_bits( unsigned size, SetOption option = SET_NOW ) = 0;

      /** Returns the flow control method. */
      virtual FlowControl flow_control() = 0;

      /**
       * Sets the flow control flags according to the flow control method specified.
       * If FLOW_CUSTOM is specified, no change will occur.
       */
      virtual void set_flow_control( FlowControl flowcontrol, SetOption option = SET_NOW ) = 0;

      /** True if the hardware carrier detect (DCD) is enabled, false otherwise. */
      virtual bool is_carrier_detect_enabled() = 0;

      /**
       * Enables or disables a hardware carrier detect.
       * If hardware carrier detect is enabled and this is a blocking TTY, an open will
       * block until a DCD signal is received from the hardware (useful for modems).
       * If this is not enabled, the TTY will behave as if the DCD is always asserted.
       * The hardware carrier flag should usually be turned on for modems and off for
       * local devices.
       */
      virtual void enable_carrier_detect( bool enable = true, SetOption option = SET_NOW ) = 0;

      /**
       * True if the receive side is enabled, false otherwise.
       * If false, this TTY cannot receive characters.
       */
      virtual bool is_receiver_enabled() = 0;

      /**
       * If true the receiver will be enabled, and if false it will be disabled
       * and no characters can be received.
       */
      virtual void enable_receiver( bool enable = true, SetOption option = SET_NOW ) = 0;

      virtual void open() = 0;

      /** Opens the TTY in the specified mode, storing the original terminal mode, to be restored on close. */
      virtual void open( const std::string& name, ReadWrite mode = READ_WRITE_UNCHANGED ) = 0;

      /** Restores the original terminal modes on the tty */
      virtual void close( bool force = false ) = 0;

      /**
       * Injects the input buffer into the input queue
       */
      virtual void inject( const std::string& buffer ) = 0;

      /** True if the terminal will be reset to original characteristics on close. */
      virtual bool is_reset_on_close_enabled() = 0;

      /** Set to true if the terminal should reset to original characteristics on close. */
      virtual void enable_reset_on_close( bool reset = true ) = 0;

      virtual sigc::signal<void>& signal_input_speed_changed() = 0;
      virtual sigc::signal<void>& signal_output_speed_changed() = 0;
      virtual sigc::signal<void>& signal_parity_changed() = 0;
      virtual sigc::signal<void>& signal_byte_size_changed() = 0;
      virtual sigc::signal<void>& signal_stop_bits_changed() = 0;
      virtual sigc::signal<void>& signal_flow_control_changed() = 0;
      virtual sigc::signal<void>& signal_carrier_detect_changed() = 0;
      virtual sigc::signal<void>& signal_receiver_changed() = 0;

  };

}

#endif
